def calc_line(p0, p1):
    """use Bresenham's algorithm to calculate a set of closed points forming a line

    Returns an array whose elements are the points of the line.

    Original calc_line code retrieved from:
    http://en.literateprograms.org/Bresenham's_line_algorithm_(Python)?oldid=16281
    """
    origin = x0, y0 = int(p0[0]), int(p0[1])
    x1, y1 = int(p1[0]), int(p1[1])
    points = []
    steep = abs(y1 - y0) > abs(x1 - x0)
    if steep:
        x0, y0 = y0, x0
        x1, y1 = y1, x1
    if x0 > x1:
        x0, x1 = x1, x0
        y0, y1 = y1, y0
    if y0 < y1:
        ystep = 1
    else:
        ystep = -1
    deltax = x1 - x0
    deltay = abs(y1 - y0)
    error = -deltax / 2
    y = y0
    for x in range(x0, x1 + 1):  # add 1 to x1 so the range includes x1
        if steep:
            points.append((y, x))
        else:
            points.append((x, y))
        error = error + deltay
        if error > 0:
            y = y + ystep
            error = error - deltax
    if origin != points[0]:
        points.reverse()
    return points
